@@ -6,23 +6,23 @@ useful when running tests on a continuous integration server.
 If the platform supports SIGALRM this is used to raise an exception in
 the test, otherwise os._exit(1) is used.
 """
-
 import os
 import signal
 import sys
 import threading
 import traceback
 from collections import namedtuple
+from distutils.version import StrictVersion
 
 import py
 import pytest
 
 
-HAVE_SIGALRM = hasattr(signal, 'SIGALRM')
+HAVE_SIGALRM = hasattr(signal, "SIGALRM")
 if HAVE_SIGALRM:
-    DEFAULT_METHOD = 'signal'
+    DEFAULT_METHOD = "signal"
 else:
-    DEFAULT_METHOD = 'thread'
+    DEFAULT_METHOD = "thread"
 TIMEOUT_DESC = """
 Timeout in seconds before dumping the stacks.  Default is 0 which
 means no timeout.
@@ -38,44 +38,48 @@ function body, ignoring the time it takes when evaluating any fixtures
 used in the test.
 """.strip()
 
-Settings = namedtuple('Settings', ['timeout', 'method', 'func_only'])
+Settings = namedtuple("Settings", ["timeout", "method", "func_only"])
 
 
 @pytest.hookimpl
 def pytest_addoption(parser):
     """Add options to control the timeout plugin"""
     group = parser.getgroup(
-        'timeout', 'Interrupt test run and dump stacks of all threads after '
-        'a test times out')
-    group.addoption('--timeout',
-                    type=float,
-                    help=TIMEOUT_DESC)
-    group.addoption('--timeout_method',
-                    action='store',
-                    choices=['signal', 'thread'],
-                    help='Deprecated, use --timeout-method')
-    group.addoption('--timeout-method',
-                    dest='timeout_method',
-                    action='store',
-                    choices=['signal', 'thread'],
-                    help=METHOD_DESC)
-    parser.addini('timeout', TIMEOUT_DESC)
-    parser.addini('timeout_method', METHOD_DESC)
-    parser.addini('timeout_func_only', FUNC_ONLY_DESC, type='bool')
+        "timeout",
+        "Interrupt test run and dump stacks of all threads after a test times out",
+    )
+    group.addoption("--timeout", type=float, help=TIMEOUT_DESC)
+    group.addoption(
+        "--timeout_method",
+        action="store",
+        choices=["signal", "thread"],
+        help="Deprecated, use --timeout-method",
+    )
+    group.addoption(
+        "--timeout-method",
+        dest="timeout_method",
+        action="store",
+        choices=["signal", "thread"],
+        help=METHOD_DESC,
+    )
+    parser.addini("timeout", TIMEOUT_DESC)
+    parser.addini("timeout_method", METHOD_DESC)
+    parser.addini("timeout_func_only", FUNC_ONLY_DESC, type="bool")
 
 
 @pytest.hookimpl
 def pytest_configure(config):
     # Register the marker so it shows up in --markers output.
     config.addinivalue_line(
-        'markers',
-        'timeout(timeout, method=None, func_only=False): Set a timeout, timeout '
-        'method and func_only evaluation on just one test item.  The first '
-        'argument, *timeout*, is the timeout in seconds while the keyword, '
-        '*method*, takes the same values as the --timeout_method option. The '
-        '*func_only* keyword, when set to True, defers the timeout evaluation '
-        'to only the test function body, ignoring the time it takes when '
-        'evaluating any fixrures used in the test.')
+        "markers",
+        "timeout(timeout, method=None, func_only=False): Set a timeout, timeout "
+        "method and func_only evaluation on just one test item.  The first "
+        "argument, *timeout*, is the timeout in seconds while the keyword, "
+        "*method*, takes the same values as the --timeout_method option. The "
+        "*func_only* keyword, when set to True, defers the timeout evaluation "
+        "to only the test function body, ignoring the time it takes when "
+        "evaluating any fixrures used in the test.",
+    )
 
     settings = get_env_settings(config)
     config._env_timeout = settings.timeout
@@ -83,13 +87,12 @@ def pytest_configure(config):
     config._env_timeout_func_only = settings.func_only
 
 
-
 @pytest.hookimpl(hookwrapper=True)
 def pytest_runtest_protocol(item):
     func_only = get_func_only_setting(item)
     if func_only is False:
         timeout_setup(item)
-    outcome = yield
+    yield
     if func_only is False:
         timeout_teardown(item)
 
@@ -99,7 +102,7 @@ def pytest_runtest_call(item):
     func_only = get_func_only_setting(item)
     if func_only is True:
         timeout_setup(item)
-    outcome = yield
+    yield
     if func_only is True:
         timeout_teardown(item)
 
@@ -107,10 +110,14 @@ def pytest_runtest_call(item):
 @pytest.hookimpl(tryfirst=True)
 def pytest_report_header(config):
     if config._env_timeout:
-        return ["timeout: %ss\ntimeout method: %s\ntimeout func_only: %s" %
-                (config._env_timeout,
-                 config._env_timeout_method,
-                 config._env_timeout_func_only)]
+        return [
+            "timeout: %ss\ntimeout method: %s\ntimeout func_only: %s"
+            % (
+                config._env_timeout,
+                config._env_timeout_method,
+                config._env_timeout_func_only,
+            )
+        ]
 
 
 @pytest.hookimpl(tryfirst=True)
@@ -125,6 +132,8 @@ def pytest_enter_pdb():
     # need another way to signify that the timeout should not be performed.
     global SUPPRESS_TIMEOUT
     SUPPRESS_TIMEOUT = True
+
+
 SUPPRESS_TIMEOUT = False
 
 
@@ -133,22 +142,27 @@ def timeout_setup(item):
     params = get_params(item)
     if params.timeout is None or params.timeout <= 0:
         return
-    if params.method == 'signal':
+    if params.method == "signal":
+
         def handler(signum, frame):
             __tracebackhide__ = True
             timeout_sigalrm(item, params.timeout)
+
         def cancel():
             signal.setitimer(signal.ITIMER_REAL, 0)
             signal.signal(signal.SIGALRM, signal.SIG_DFL)
+
         item.cancel_timeout = cancel
         signal.signal(signal.SIGALRM, handler)
         signal.setitimer(signal.ITIMER_REAL, params.timeout)
-    elif params.method == 'thread':
-        timer = threading.Timer(params.timeout, timeout_timer,
-                                (item, params.timeout))
+    elif params.method == "thread":
+        timer = threading.Timer(params.timeout, timeout_timer, (item, params.timeout))
+        timer.name = "%s %s" % (__name__, item.nodeid)
+
         def cancel():
             timer.cancel()
             timer.join()
+
         item.cancel_timeout = cancel
         timer.start()
 
@@ -159,46 +173,46 @@ def timeout_teardown(item):
     # (as is the case when using the pytest.mark.skipif marker) we
     # may be called without our setup counterpart having been
     # called.
-    cancel = getattr(item, 'cancel_timeout', None)
+    cancel = getattr(item, "cancel_timeout", None)
     if cancel:
         cancel()
 
 
 def get_env_settings(config):
-    timeout = config.getvalue('timeout')
+    timeout = config.getvalue("timeout")
     if timeout is None:
         timeout = _validate_timeout(
-            os.environ.get('PYTEST_TIMEOUT'),
-            'PYTEST_TIMEOUT environment variable')
+            os.environ.get("PYTEST_TIMEOUT"), "PYTEST_TIMEOUT environment variable"
+        )
     if timeout is None:
-        ini = config.getini('timeout')
+        ini = config.getini("timeout")
         if ini:
-            timeout = _validate_timeout(ini, 'config file')
+            timeout = _validate_timeout(ini, "config file")
 
-    method = config.getvalue('timeout_method')
+    method = config.getvalue("timeout_method")
     if method is None:
-        ini = config.getini('timeout_method')
+        ini = config.getini("timeout_method")
         if ini:
-            method = _validate_method(ini, 'config file')
+            method = _validate_method(ini, "config file")
     if method is None:
         method = DEFAULT_METHOD
 
-    func_only = config.getini('timeout_func_only')
+    func_only = config.getini("timeout_func_only")
     if func_only == []:
         # No value set
         func_only = None
     if func_only is not None:
-        func_only = _validate_func_only(func_only, 'config file')
+        func_only = _validate_func_only(func_only, "config file")
     return Settings(timeout, method, func_only or False)
 
 
 def get_func_only_setting(item):
     """Return the func_only setting for an item"""
     func_only = None
-    marker = item.get_closest_marker('timeout')
+    marker = item.get_closest_marker("timeout")
     if marker:
         settings = get_params(item, marker=marker)
-        func_only = _validate_func_only(settings.func_only, 'marker')
+        func_only = _validate_func_only(settings.func_only, "marker")
     if func_only is None:
         func_only = item.config._env_timeout_func_only
     if func_only is None:
@@ -210,12 +224,12 @@ def get_params(item, marker=None):
     """Return (timeout, method) for an item"""
     timeout = method = func_only = None
     if not marker:
-        marker = item.get_closest_marker('timeout')
+        marker = item.get_closest_marker("timeout")
     if marker is not None:
-        settings = _parse_marker(item.get_closest_marker(name='timeout'))
-        timeout = _validate_timeout(settings.timeout, 'marker')
-        method = _validate_method(settings.method, 'marker')
-        func_only = _validate_func_only(settings.func_only, 'marker')
+        settings = _parse_marker(item.get_closest_marker(name="timeout"))
+        timeout = _validate_timeout(settings.timeout, "marker")
+        method = _validate_method(settings.method, "marker")
+        func_only = _validate_func_only(settings.func_only, "marker")
     if timeout is None:
         timeout = item.config._env_timeout
     if method is None:
@@ -232,30 +246,27 @@ def _parse_marker(marker):
     could still be bogus and even the wrong type.
     """
     if not marker.args and not marker.kwargs:
-        raise TypeError('Timeout marker must have at least one argument')
+        raise TypeError("Timeout marker must have at least one argument")
     timeout = method = func_only = NOTSET = object()
     for kw, val in marker.kwargs.items():
-        if kw == 'timeout':
+        if kw == "timeout":
             timeout = val
-        elif kw == 'method':
+        elif kw == "method":
             method = val
-        elif kw == 'func_only':
+        elif kw == "func_only":
             func_only = val
         else:
-            raise TypeError(
-                'Invalid keyword argument for timeout marker: %s' % kw)
+            raise TypeError("Invalid keyword argument for timeout marker: %s" % kw)
     if len(marker.args) >= 1 and timeout is not NOTSET:
-        raise TypeError(
-            'Multiple values for timeout argument of timeout marker')
+        raise TypeError("Multiple values for timeout argument of timeout marker")
     elif len(marker.args) >= 1:
         timeout = marker.args[0]
     if len(marker.args) >= 2 and method is not NOTSET:
-        raise TypeError(
-            'Multiple values for method argument of timeout marker')
+        raise TypeError("Multiple values for method argument of timeout marker")
     elif len(marker.args) >= 2:
         method = marker.args[1]
     if len(marker.args) > 2:
-        raise TypeError('Too many arguments for timeout marker')
+        raise TypeError("Too many arguments for timeout marker")
     if timeout is NOTSET:
         timeout = None
     if method is NOTSET:
@@ -272,15 +283,15 @@ def _validate_timeout(timeout, where):
     try:
         return float(timeout)
     except ValueError:
-        raise ValueError('Invalid timeout %s from %s' % (timeout, where))
+        raise ValueError("Invalid timeout %s from %s" % (timeout, where))
 
 
 def _validate_method(method, where):
     """Helper for get_params()"""
     if method is None:
         return None
-    if method not in ['signal', 'thread']:
-        raise ValueError('Invalid method %s from %s' % (method, where))
+    if method not in ["signal", "thread"]:
+        raise ValueError("Invalid method %s from %s" % (method, where))
     return method
 
 
@@ -289,7 +300,7 @@ def _validate_func_only(func_only, where):
     if func_only is None:
         return False
     if not isinstance(func_only, bool):
-        raise ValueError('Invalid func_only value %s from %s' % (func_only, where))
+        raise ValueError("Invalid func_only value %s from %s" % (func_only, where))
     return func_only
 
 
@@ -305,11 +316,11 @@ def timeout_sigalrm(item, timeout):
     __tracebackhide__ = True
     nthreads = len(threading.enumerate())
     if nthreads > 1:
-        write_title('Timeout', sep='+')
+        write_title("Timeout", sep="+")
     dump_stacks()
     if nthreads > 1:
-        write_title('Timeout', sep='+')
-    pytest.fail('Timeout >%ss' % timeout)
+        write_title("Timeout", sep="+")
+    pytest.fail("Timeout >%ss" % timeout)
 
 
 def timeout_timer(item, timeout):
@@ -321,32 +332,31 @@ def timeout_timer(item, timeout):
     if SUPPRESS_TIMEOUT:
         return
     try:
-        capman = item.config.pluginmanager.getplugin('capturemanager')
+        capman = item.config.pluginmanager.getplugin("capturemanager")
         if capman:
-            if pytest.__version__ >= '3.7.3':
+            pytest_version = StrictVersion(pytest.__version__)
+            if pytest_version >= StrictVersion("3.7.3"):
                 capman.suspend_global_capture(item)
                 stdout, stderr = capman.read_global_capture()
-            elif pytest.__version__ >= '3.3':
-                stdout, stderr = capman.suspend_global_capture(item)
             else:
-                stdout, stderr = capman.suspendcapture(item)
+                stdout, stderr = capman.suspend_global_capture(item)
         else:
             stdout, stderr = None, None
-        write_title('Timeout', sep='+')
-        caplog = item.config.pluginmanager.getplugin('_capturelog')
-        if caplog and hasattr(item, 'capturelog_handler'):
+        write_title("Timeout", sep="+")
+        caplog = item.config.pluginmanager.getplugin("_capturelog")
+        if caplog and hasattr(item, "capturelog_handler"):
             log = item.capturelog_handler.stream.getvalue()
             if log:
-                write_title('Captured log')
+                write_title("Captured log")
                 write(log)
         if stdout:
-            write_title('Captured stdout')
+            write_title("Captured stdout")
             write(stdout)
         if stderr:
-            write_title('Captured stderr')
+            write_title("Captured stderr")
             write(stderr)
         dump_stacks()
-        write_title('Timeout', sep='+')
+        write_title("Timeout", sep="+")
     except Exception:
         traceback.print_exc()
     finally:
@@ -366,12 +376,12 @@ def dump_stacks():
                 thread_name = t.name
                 break
         else:
-            thread_name = '<unknown>'
-        write_title('Stack of %s (%s)' % (thread_name, thread_ident))
-        write(''.join(traceback.format_stack(frame)))
+            thread_name = "<unknown>"
+        write_title("Stack of %s (%s)" % (thread_name, thread_ident))
+        write("".join(traceback.format_stack(frame)))
 
 
-def write_title(title, stream=None, sep='~'):
+def write_title(title, stream=None, sep="~"):
     """Write a section title
 
     If *stream* is None sys.stderr will be used, *sep* is used to
@@ -381,10 +391,10 @@ def write_title(title, stream=None, sep='~'):
         stream = sys.stderr
     width = py.io.get_terminal_width()
     fill = int((width - len(title) - 2) / 2)
-    line = ' '.join([sep * fill, title, sep * fill])
+    line = " ".join([sep * fill, title, sep * fill])
     if len(line) < width:
         line += sep * (width - len(line))
-    stream.write('\n' + line + '\n')
+    stream.write("\n" + line + "\n")
 
 
 def write(text, stream=None):
